      subroutine spinorkk(p,mass,ip,io,imc,phi,u,npol)
c----------------------------------------------------------------------c
c Returns helicity spinors u(4,npol) of four momentum p and mass m for
c a particle/an antiparticle (ip=1/-1), incoming/outgoing (io=-1/1)
c in the Weyl representation.
c The spinors are defined as in F. Jegerlehner, K. Ko\l odziej, 
c Eur. Phys. J. C12 (2000) 77, [hep-ph/9907229]
c             u(:,1) corresponds to sigma = -1,
c             u(:,2) corresponds to sigma = +1.
c             npol = 1 for a neutrino/antineutrino,
c                    2 for a charged fermion/antifermion.
c If imc=1 then polarizations of a charged fermion are summed
c        exp(i*phi)*u(:,1) + exp(-i*phi)*u(:,2)
c else
c   2 or 1 polarized spinors are returned for a charged fermion or 
c   neutrino.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 20 March, 2005;  modified on 22 March, 2006 (interface changed),
c                              6 May (interface changed again)
c                              19 October, 2007 (MC summing over 
c                                            polarizations implemented)
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0)
      integer :: ip,io,ib,imc,npol
      real(dp) :: p(0:3),mass,phi,pt,pp,ct,cf,sf,cth,sth,epp,emp
      complex(dp), parameter :: ocom=(0.d0,0.d0)
      complex(dp) :: u(4,npol),uu(4,2),chi(2,2),eppcom,empcom,epif

      if (ip == 1) then
       if (io == -1) then
        ib=0
       else if (io == 1) then
        ib=1
       else
        write(*,*)'spinorkk.f: wrong value of io=',io
        stop
       end if
      else if (ip == -1) then
       if (io == -1) then
        ib=1
       else if (io == 1) then
        ib=0
       else
        write(*,*)'spinorkk.f: wrong value of io=',io
        stop
       end if
      else
       write(*,*)'spinorkk.f: wrong value of ip=',ip
       stop
      end if

      pt=sqrt(p(1)**2+p(2)**2)

      if (pt == 0.d0) then

       pp=abs(p(3))
       ct=p(3)/pp
       cf=1.d0
       sf=0.d0

      else

       pp=sqrt(pt**2+p(3)**2)
       ct=p(3)/pp
       cf=p(1)/pt
       sf=p(2)/pt

      end if


      if (mass == 0.d0) then
       epp=sqrt(2.d0*p(0))
       if (abs(ct) < 1.d0) then
        cth=sqrt((1.d0+ct)/2.d0)
        sth=sqrt((1.d0-ct)/2.d0)
        if (npol == 2 .or. imc == 1) then
         chi(:,1)=(/-sth*cmplx(cf,-sf,dp),cmplx(cth,0.d0,dp)/)
         chi(:,2)=(/chi(2,1),-conjg(chi(1,1))/)
        else
         chi(:,1)=(/-sth*cmplx(cf,-sf,dp),cmplx(cth,0.d0,dp)/)
        end if
        if (ib == 0) then
         if (ip == 1) then
          if (npol == 2 .or. imc == 1) then
           uu(:,1)=(/epp*chi(1,1),epp*chi(2,1),ocom,ocom/)
           uu(:,2)=(/ocom,ocom,epp*chi(1,2),epp*chi(2,2)/)
          else
           uu(:,1)=(/epp*chi(1,1),epp*chi(2,1),ocom,ocom/)
          end if
         else ! ip=-1
          if (npol == 2 .or. imc == 1) then
           uu(:,1)=(/ocom,ocom,-epp*chi(1,2),-epp*chi(2,2)/)
           uu(:,2)=(/-epp*chi(1,1),-epp*chi(2,1),ocom,ocom/)
          else
           uu(:,1)=(/-epp*chi(1,1),-epp*chi(2,1),ocom,ocom/)
          end if
         end if ! ip
        else ! ib=1
         if (npol == 1 .and. imc /= 1) then
          chi(:,1)=(/-sth*cmplx(cf,sf,dp),cmplx(cth,0.d0,dp)/)
         end if
         if (ip == 1) then
          if (npol == 2 .or. imc == 1) then
           uu(:,1)=(/ocom,ocom,-epp*chi(2,2),epp*chi(1,2)/)
           uu(:,2)=(/epp*chi(2,1),-epp*chi(1,1),ocom,ocom/)
          else
           uu(:,1)=(/ocom,ocom,epp*chi(1,1),epp*chi(2,1)/)
          end if
         else ! ip=-1
          if (npol == 2 .or. imc == 1) then
           uu(:,1)=(/-epp*chi(2,1),epp*chi(1,1),ocom,ocom/)
           uu(:,2)=(/ocom,ocom,epp*chi(2,2),-epp*chi(1,2)/)
          else
           uu(:,1)=(/ocom,ocom,-epp*chi(1,1),-epp*chi(2,1)/)
          end if
         end if ! ip
        end if ! ib
       else ! abs(ct)=1
        eppcom=cmplx(epp,0.d0,dp)
        if (ct == 1.d0) then
         if (ib == 0) then
          if (ip == 1) then
           if (npol == 2 .or. imc == 1) then
            uu(:,1)=(/ocom,eppcom,ocom,ocom/)
            uu(:,2)=(/ocom,ocom,eppcom,ocom/)
           else
            uu(:,1)=(/ocom,eppcom,ocom,ocom/)
           end if
          else ! ip=-1
           if (npol == 2 .or. imc == 1) then
            uu(:,1)=(/ocom,ocom,-eppcom,ocom/)
            uu(:,2)=(/ocom,-eppcom,ocom,ocom/)
           else
            uu(:,1)=(/ocom,-eppcom,ocom,ocom/)
           end if
          end if ! ip
         else ! ib=1
          if (ip == 1) then
           if (npol == 2 .or. imc == 1) then
            uu(:,1)=(/ocom,ocom,ocom,eppcom/)
            uu(:,2)=(/eppcom,ocom,ocom,ocom/)
           else
            uu(:,1)=(/ocom,ocom,ocom,eppcom/)
           end if
          else ! ip=-1
           if (npol == 2 .or. imc == 1) then
            uu(:,1)=(/-eppcom,ocom,ocom,ocom/)
            uu(:,2)=(/ocom,ocom,ocom,-eppcom/)
           else
            uu(:,1)=(/ocom,ocom,ocom,-eppcom/)
           end if
          end if ! ip
         end if ! ib
        else ! ct=-1
         if (ib == 0) then
          if (ip == 1) then
           if (npol == 2 .or. imc == 1) then
            uu(:,1)=(/eppcom,ocom,ocom,ocom/)
            uu(:,2)=(/ocom,ocom,ocom,-eppcom/)
           else
            uu(:,1)=(/eppcom,ocom,ocom,ocom/)
           end if 
          else ! ip=-1
           if (npol == 2 .or. imc == 1) then
            uu(:,1)=(/ocom,ocom,ocom,eppcom/)
            uu(:,2)=(/-eppcom,ocom,ocom,ocom/)
           else 
            uu(:,1)=(/-eppcom,ocom,ocom,ocom/)
           end if 
          end if ! ip
         else ! ib=1
          if (ip == 1) then
           if (npol == 2 .or. imc == 1) then
            uu(:,1)=(/ocom,ocom,eppcom,ocom/)
            uu(:,2)=(/ocom,-eppcom,ocom,ocom/)
           else
            uu(:,1)=(/ocom,ocom,eppcom,ocom/)
           end if
          else ! ip=-1
           if (npol == 2 .or. imc == 1) then
            uu(:,1)=(/ocom,eppcom,ocom,ocom/)
            uu(:,2)=(/ocom,ocom,-eppcom,ocom/)
           else
            uu(:,1)=(/ocom,ocom,-eppcom,ocom/)
           end if
          end if ! ip
         end if ! ib
        end if ! ct
       end if ! abs(ct)
      else ! mass
       epp=sqrt(p(0)+pp)
       emp=mass/epp
       if (abs(ct) < 1.d0) then
        cth=sqrt((1.d0+ct)/2.d0)
        sth=sqrt((1.d0-ct)/2.d0)
        chi(:,1)=(/-sth*cmplx(cf,-sf,dp),cmplx(cth,0.d0,dp)/)
        chi(:,2)=(/chi(2,1),-conjg(chi(1,1))/)
        if (ib == 0) then
         if (ip == 1) then
          uu(:,1)=(/epp*chi(1,1),epp*chi(2,1),
     &                             emp*chi(1,1),emp*chi(2,1)/)
          uu(:,2)=(/emp*chi(1,2),emp*chi(2,2),
     &                             epp*chi(1,2),epp*chi(2,2)/)
         else ! ip=-1
          uu(:,1)=(/emp*chi(1,2),emp*chi(2,2),
     &                            -epp*chi(1,2),-epp*chi(2,2)/)
          uu(:,2)=(/-epp*chi(1,1),-epp*chi(2,1),
     &                             emp*chi(1,1),emp*chi(2,1)/)
         end if ! ip
        else ! ib=1
         if (ip == 1) then
          uu(:,1)=(/-emp*chi(2,2),emp*chi(1,2),
     &                             -epp*chi(2,2),epp*chi(1,2)/)
          uu(:,2)=(/epp*chi(2,1),-epp*chi(1,1),
     &                              emp*chi(2,1),-emp*chi(1,1)/)
         else ! ip=-1
          uu(:,1)=(/-epp*chi(2,1),epp*chi(1,1),
     &                              emp*chi(2,1),-emp*chi(1,1)/)
          uu(:,2)=(/-emp*chi(2,2),emp*chi(1,2),
     &                              epp*chi(2,2),-epp*chi(1,2)/)
         end if ! ip
        end if ! ib
       else ! abs(ct)=1
        eppcom=cmplx(epp,0.d0,dp)
        empcom=cmplx(emp,0.d0,dp)
        if (ct == 1.d0) then
         if (ib == 0) then
          if (ip == 1) then
           uu(:,1)=(/ocom,eppcom,ocom,empcom/)
           uu(:,2)=(/empcom,ocom,eppcom,ocom/)
          else ! ip=-1
           uu(:,1)=(/empcom,ocom,-eppcom,ocom/)
           uu(:,2)=(/ocom,-eppcom,ocom,empcom/)
          end if ! ip
         else ! ib=1
          if (ip == 1) then
           uu(:,1)=(/ocom,empcom,ocom,eppcom/)
           uu(:,2)=(/eppcom,ocom,empcom,ocom/)
          else ! ip=-1
           uu(:,1)=(/-eppcom,ocom,empcom,ocom/)
           uu(:,2)=(/ocom,empcom,ocom,-eppcom/)
          end if ! ip
         end if ! ib
        else ! ct=-1
         if (ib == 0) then
          if (ip == 1) then
           uu(:,1)=(/eppcom,ocom,empcom,ocom/)
           uu(:,2)=(/ocom,-empcom,ocom,-eppcom/)
          else ! ip=-1
           uu(:,1)=(/ocom,-empcom,ocom,eppcom/)
           uu(:,2)=(/-eppcom,ocom,empcom,ocom/)
          end if ! ip
         else ! ib=1
          if (ip == 1) then
           uu(:,1)=(/empcom,ocom,eppcom,ocom/)
           uu(:,2)=(/ocom,-eppcom,ocom,-empcom/)
          else ! ip=-1
           uu(:,1)=(/ocom,eppcom,ocom,-empcom/)
           uu(:,2)=(/empcom,ocom,-eppcom,ocom/)
          end if ! ip
         end if ! ib
        end if ! ct
       end if ! abs(ct)
      end if ! mass

      if (imc == 1) then
       epif=cmplx(cos(phi),sin(phi),dp)
       u(:,1)=epif*uu(:,1)+conjg(epif)*uu(:,2)
c       u(:,1)=cmplx(cos(phi),sin(phi),dp)*uu(:,1)+uu(:,2)
      else
       if (npol == 1) then
        u(:,1)=uu(:,1)
       else
        u=uu
       end if
      end if

      end subroutine

